# encoding: utf-8

from __future__ import unicode_literals

from abc import ABCMeta
from abc import abstractmethod
from abc import abstractproperty
from collections import Mapping

from .. import SKIP_ERRORS
from ..logging import Logger
from ..parsing import ExcelParser


class IConfig(object):
    """Интерфейс "Конфигурация"."""

    @abstractproperty
    def mode(self):
        """Режим загрузки.

        :returns: STRICT, SKIP_ERRORS или DRY_RUN
        :rtype: int
        """

    @abstractproperty
    def parser(self):
        """Класс парсера.

        :returns: Класс, реализующий интерфейс парсера
        :rtype: :py:class:data_import.parsing.IParser
        """

    @abstractproperty
    def loader(self):
        """Класс загрузчика.

        :returns: Класс, реализующий интерфейс загрузчика
        :rtype: :py:class:data_import.loading.ILoader
        """

    @abstractproperty
    def logger(self):
        """Класс логгера.

        :returns: Класс, реализующий интерфейс логгера
        :rtype: :py:class:data_import.logging.ILogger
        """

    @abstractproperty
    def rules(self):
        """Класс набора правил.

        :rtype: :py:class:data_import.configuration.Rules
        """

    @abstractproperty
    def need_confirm(self):
        """Запрос подтверждения.

        :rtype: bool
        """


class IRegistry(object):
    """Интерфейс реестра конфигураций."""

    __metaclass__ = ABCMeta

    @abstractmethod
    def get_configs(self, ext=None):
        """Получение зарегистрированных конфигураций.

        Если указан параметр `ext`, вернёт конфигурации, поддерживающие импорт
        данного типа файла.

        :param basestring ext: Расширение имени файла
        """

    @abstractmethod
    def register(self, config):
        """Регистрация конфигурации в реестре.

        :param config: Класс конфигурации
        """


class AbstractExcelConfig(IConfig):
    """Базовый класс конфигурации импорта Excel."""

    mode = SKIP_ERRORS
    need_confirm = False
    parser = ExcelParser
    logger = Logger


class Rules(Mapping):
    """Набор правил импорта."""

    def _keys(self):
        parent_keys = dir(Mapping)
        for key in dir(self):
            if not key.startswith('_') and key not in parent_keys:
                yield key

    def __iter__(self):
        return self._keys()

    def __len__(self):
        return sum(1 for _ in self._keys())

    def __getitem__(self, key):
        return getattr(self, key)


class Registry(IRegistry):
    """Реестр конфигураций загрузчиков."""

    def __init__(self, configs=None):
        self._configs = []

        for config in configs or ():
            self.register(config)

    def get_configs(self, ext=None):
        """Получение зарегистрированных конфигураций.

        Если указан параметр `ext`, вернёт конфигурации, поддерживающие импорт
        данного типа файла.

        :param basestring ext: Расширение имени файла
        """
        if ext:
            return tuple(
                config for config in self._configs
                if ext in config.parser.extensions
            )
        return self._configs

    def register(self, config):
        self._configs.append(config)


# Инстанцирование реестра происходит на стороне проекта
registry = None
