# encoding: utf-8
from __future__ import unicode_literals

from os import path
from subprocess import call

from celery.app.task import Task
from data_import import import_file
from data_import.logging import Logger

from .exceptions import ExtractError
from .utils import files_by_priority
from .utils import prepare_dir


class BulkImportTaskMixin(Task):
    """Примесь массового импорта для задач Celery.

    Используется, например, совместно с классом задач, формирующих реестр
    фоновых задач.

    .. code-block:: python

        from ssuz.async.tasks import AsyncTask


        class BulkImportTask(BulkImportTaskMixin, AsyncTask):

            LOG_TIME_FORMAT = u'%Y-%m-%d %H:%M:%s'

            def _get_parser_params(self, file_path, config):
                return dict(
                    skip_sheets=('Справочник', )
                )

            def run(self, *args, **kwargs):
                _, errors = super(BulkImportTask, self).run(*args, **kwargs)
                if not errors:
                    self.set_progress(
                        progress="Успешно",
                        values=dict(
                            results="Время окончания: {}".format(
                                datetime.datetime.now().strftime(
                                    self.LOG_TIME_FORMAT
                                )
                            )
                        ),
                        task_state=SUCCESS
                    )
                else:
                    message = render_to_string(
                        'bulk-errors.html', context=dict(errors=errors)
                    )
                    self.set_progress(
                        progress='Ошибки при сохранении',
                        values=dict(results=message),
                        task_state=FAILURE
                    )
                return self.state
    """

    abstract = True

    logger_cls = Logger

    MSG_TEMPLATE_NOT_DETECTED = 'шаблон не распознан.'

    def _unzip(self, source, destination):
        return_code = call(
            ('unzip', '-q', '-j', '-o', source, '-d', destination)
        )
        if return_code > 1:
            raise ExtractError('Ошибка при распаковке архива')

    extract_methods = {
        'zip': _unzip,
    }

    def _extract(self, source, destination):
        """Распаковка файла."""

        file_ext = path.splitext(source)[1].strip('.')
        try:
            self.extract_methods[file_ext](self, source, destination)
        except LookupError:
            raise ExtractError('Ошибка при распаковке архива')

    def _get_logger(self):
        return self.logger_cls()

    def _get_parser_params(self, file_path, config):
        return {}

    def _import_file(self, file_):
        source, destination = prepare_dir(path.realpath(file_))
        self._extract(source, destination)

        results = []

        # Загрузка в порядке от высшего к низшему
        for _, config, file_path in files_by_priority(destination):
            filename = path.basename(file_path)
            if not config:
                results.append(
                    (filename, (self.MSG_TEMPLATE_NOT_DETECTED, ))
                )
                continue

            parser_params = self._get_parser_params(file_path, config)

            try:
                log = import_file(file_path, config, parser_params)

            finally:
                results.append((filename, log))

        return results

    def run(self, *args, **kwargs):
        result = super(BulkImportTaskMixin, self).run(*args, **kwargs)
        log = self._import_file(kwargs['file_path'])
        return result, log
