# encoding: utf-8
from __future__ import unicode_literals

from django import VERSION
from django.core.exceptions import MultipleObjectsReturned
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Model
from django.db.models.fields import FieldDoesNotExist
from django.db.models.fields.related import ManyToOneRel
from m3_django_compat import get_model
import six

from .. import AbstractLoader
from ... import ImportException
from ..utils import is_nested
from .utils import choice_value
from .utils import group_values


class ModelLoader(AbstractLoader):
    """Загрузчик данных в модель Django ORM."""

    model = None

    get_method_name_format = 'get_{0}'

    # Режим получения объекта модели по параметрам
    MODE_GET = 0  # Только выборка, без создания или обновления
    MODE_UPDATE = 1  # Обновление
    MODE_CREATE = 2  # Выборка или создание

    mode = MODE_UPDATE

    match_fields = ()

    def __init__(self, logger, params=None):
        super(ModelLoader, self).__init__(logger, params)

        if isinstance(self.model, six.string_types):
            self.model = get_model(*self.model.split('.'))

        elif not issubclass(self.model, Model):
            raise ValueError(
                'Модель указывается строкой или классом модели'
            )

    def _get_model_manager(self):
        """Возвращается базовый менеджер для доступа ко всем записям."""
        # pylint: disable=W0212
        current_version = VERSION[:2]

        if current_version < (1, 10):
            return self.model._default_manager

        return self.model._meta.default_manager

    def _select_method(self):
        """Выбор метода определения объекта модели."""
        manager = self._get_model_manager()
        methods = {
            self.MODE_GET: manager.get,
            self.MODE_UPDATE: manager.get_or_create,
            self.MODE_CREATE: manager.create
        }
        return methods[self.mode]

    def _match_instance(self, data):
        """Определение объекта модели по полученным данным."""

        method = self._select_method()
        if self.mode == self.MODE_CREATE:
            ident_data = data
        else:
            ident_data = {
                attname: data[attname] for attname in self.match_fields
            }
        if self.mode == self.MODE_UPDATE:
            ident_data.update(defaults=data)

        try:
            result = method(**ident_data)
        except ObjectDoesNotExist:
            raise ImportException(
                '{} не существует'.format(self.model._meta.verbose_name)
            )

        if not isinstance(result, tuple):
            created = self.mode is self.MODE_CREATE
            result = result, created

        return result

    def _fetch_model_object(self, field, value, data):

        method_name = self.get_method_name_format.format(field.name)
        method = getattr(self, method_name, None)

        if method is not None:
            try:
                return method(data)
            except MultipleObjectsReturned:
                raise ImportException(
                    '{} - найдено несколько записей'.format(
                        field.verbose_name
                    )
                )

        try:
            related_model = field.related_model
        except AttributeError:
            related_model = field.related.model

        if is_nested(value):
            attname, value = value.items()[0]
            nested_instance = self._fetch_model_object(
                # pylint: disable=W0212
                related_model._meta.get_field(attname), value, data
            )
            query_params = {field.attname: nested_instance}

        else:
            query_params = value

        try:
            instance = related_model.objects.get(**query_params)
        except ObjectDoesNotExist:
            raise ImportException(
                '{} не существует'.format(self.model._meta.verbose_name)
            )
        except MultipleObjectsReturned:
            raise ImportException(
                '{} - найдено несколько записей'.format(
                    self.model._meta.verbose_name
                )
            )

        return instance

    def _prepare_values(self, values):
        """Получение связанных объектов, если поле ссылается на модель."""
        options = self.model._meta  # pylint: disable=W0212

        for attname, value in values.items():
            try:
                field = options.get_field(attname)
            except FieldDoesNotExist:
                continue

            if isinstance(field, ManyToOneRel):
                continue

            if isinstance(value, dict):  # relation
                values[attname] = self._fetch_model_object(
                    field, value, values
                )

            else:  # simple value
                if getattr(field, 'choices', []) and value:
                    values[attname] = choice_value(field, value)

        return values

    def _update_instance(self, instance, data):
        """Обновление объекта модели данными из словаря."""

        for attname, value in six.iteritems(data):
            setattr(instance, attname, value)

        instance.full_clean()
        instance.save()

    def _load(self, row):
        """Загрузка строки в БД."""
        row.values = self._prepare_values(group_values(row.values))

        instance, created = self._match_instance(row.values)
        if self.mode is not self.MODE_GET:
            self._update_instance(instance, row.values)

        return instance, created
