# encoding: utf-8
from __future__ import unicode_literals

from collections import OrderedDict

import six


def choice_value(field, value):
    """Получение значения для поля по значению из field.choices."""

    for choice, val in field.choices:
        if value == val:
            return choice

    raise KeyError(value)


def is_lookup(string):
    """Определяет, является ли строка lookup'ом.

    :param basestring string: проверяемая строка
    :returns: bool -- результат проверки
    """

    return '__' in string


def lookup_to_dict(data, lookup, value):
    """Обновление словаря данных lookup-строкой cо значением.

    :param dict data: словарь данных
    :param basestring lookup: lookup-строка
    :param value: значение поля модели

    .. code-block:: python

        data = {}
        lookup = 'unit__speciality__code'
        value = '000'

        lookup_to_dict(data, lookup, value)
        {
            'unit': {
                'speciality': {
                    'code': '000'
                }
            }
        }
    """

    attname, _, rest_attrs = (
        lookup.partition('__')
    )

    data.setdefault(attname, {})

    if is_lookup(rest_attrs):
        lookup_to_dict(data[attname], rest_attrs, value)
    else:
        data[attname][rest_attrs] = value

    data.pop(lookup, None)


def dict_to_lookup(dict_, prefix=None):
    """Преобразование вложенного словаря в плоский словарь.

    Используется для передачи в методы Queryset (.filter(), .get(), etc.)

    :param dict dict_: словарь с параметрами
    :param str prefix: префикс lookup'ов

    >>> dict_ = {
        'specialization': {
            'speciality': {
                'code': 'xxx',
                'name': 'xxx'
            }
        }
    }

    >>> dict_to_lookup(dict_)
    {
        'speciality__code': 'xxx',
        'speciality__name': 'xxx'
    }

    """

    for key, value in six.iteritems(dict_):
        if isinstance(value, dict):
            return dict_to_lookup(value, key)
        else:
            result = dict_
            if prefix:
                result = {
                    u'__'.join((prefix, attr_name)): val
                    for attr_name, val in six.iteritems(dict_)
                }
            return result


def group_values(data):
    """Группировка словаря lookup'ов во вложенный словарь.

    :param dict data: Словарь lookup'ов

    >>> input_ = {
        'unit__speciality__code': '000',
        'unit__speciality__name': 'xxx',
        'unit__code': '111',
        'unit__name': 'yyy',
        'code': '222',
        'name': 'zzz'
    }
    >>> group_data(input_)
    {
        'unit': {
            'speciality': {
                'code': '000',
                'name': 'xxx',
            },
            'code': '111',
            'name': 'yyy',
        },
        'code': '222',
        'name': 'zzz',
    }
    """

    # Преобразование в словари
    for attname, value in list(six.iteritems(data)):
        if is_lookup(attname):
            lookup_to_dict(data, attname, value)

    # Группировка - сначала вложенные объекты, затем - простые значения
    result = OrderedDict()
    for key, value in list(six.iteritems(data)):
        if isinstance(value, dict):
            result[key] = data.pop(key)
    result.update(data)

    data = result

    return data