# encoding: utf-8

from __future__ import unicode_literals

import re
from abc import ABCMeta
from abc import abstractmethod
from abc import abstractproperty
from collections import OrderedDict
from datetime import date
from datetime import datetime

from six import text_type
from six.moves import zip
from xlrd import open_workbook
from xlrd import XLRDError
from xlrd.xldate import xldate_as_tuple

from .. import ImportException
from .. import ParsingError
from ..validation import Validator


class Row(object):
    """Строка данных."""

    row_number = None
    values = None

    def __unicode__(self):
        return 'Строка {0}'.format(self.row_number)

    __str__ = __unicode__


class ExcelRow(Row):
    """Строка данных при импорте из Excel."""

    def __init__(self):
        super(ExcelRow, self).__init__()
        self.sheet_name = None

    def __unicode__(self):
        return 'Лист "{0}", {1}'.format(
            self.sheet_name, super(ExcelRow, self).__unicode__()
        )

    __str__ = __unicode__


class IParser(object):
    """Интерфейс парсера."""

    __metaclass__ = ABCMeta

    row_cls = None
    validator_cls = None

    @abstractproperty
    def extensions(self):
        """Список расширений фалов, поддерживаемых парсером."""

    @abstractmethod
    def get_rows(self):
        """Возвращает коллекцию строк."""


class ExcelParser(IParser):
    """Парсер файлов Excel."""

    row_cls = ExcelRow
    validator_cls = Validator

    extensions = ('xls', 'xlsx')

    def __init__(
        self, file_path, rules, logger, skip_rows=None, skip_sheets=None
    ):
        """Создает объект.

        Если открыть файл не удалось, то выбрасывается ParsingError
        :param file_path: путь к файлу
        :type file_path: str
        :param rules: правила, которым должны соответствовать значения в файле
        :param logger: Логгер
        :type logger: ILogger
        :param skip_rows: кортеж с номерами строк, которые нужно пропустить
        :type skip_rows: tuple
        :param skip_sheets: кортеж с именами листов, которые нужно пропустить
        :type skip_sheets: tuple
        """
        self.file_path = file_path
        self.rules = rules
        self.logger = logger

        self.header_row = 0  # Номер строки шапки
        # Какие строки пропускаем
        self.skip_rows = tuple(skip_rows or ()) + (self.header_row, )

        self.skip_sheets = skip_sheets or ()
        try:
            self._workbook = open_workbook(file_path)
        except XLRDError as xlrd_error:
            raise ParsingError().with_traceback(xlrd_error.__traceback__)
        self._header_config = {rules[key]['header']: key for key in rules}

    def get_sheets(self):
        """Возвращает список листов."""
        for sheet in self._workbook.sheets():
            if sheet.name not in self.skip_sheets:
                yield sheet

    def _missed_headers(self, headers):
        """Возвращает имена столбцов отсутствующих в импортируемом файле.

        Возвращает имя столбца если оно не было найдено в файле ни по его
        полному названию, ни по регулярному выражению.
        """
        tmp_config = self._header_config.copy().items()
        for key, val in tmp_config:
            rule = self.rules[val]
            if 'regex_pattern' in rule:
                regex_pattern = self.rules[val]['regex_pattern']
                r = re.compile(regex_pattern, flags=re.IGNORECASE)
                header = tuple(filter(r.search, headers))
                if header:
                    self._header_config[header[0]] = val
                else:
                    yield key
            elif key not in headers:
                yield key

    @property
    def headers(self):
        """Имена колонок таблицы."""
        sheet = next(self.get_sheets())
        headers = [cell.value.strip() for cell in sheet.row(self.header_row)]
        missed_headers = tuple(self._missed_headers(headers))
        if missed_headers:
            raise ImportException(
                'На листе "{0}" отсутствуют столбцы: {1}'.format(
                    sheet.name,
                    ', '.join(missed_headers)
                )
            )
        return headers

    def _get_validator(self, rules):
        validator = self.validator_cls(rules)
        return validator

    def _get_row(self):
        return self.row_cls()

    def _prepare_row(self, row):
        values = getattr(row, 'values', {})

        for attname, value in values.items():
            rules = self.rules[attname]
            if 'type' in rules and not isinstance(value, rules['type']):
                try:
                    if rules['type'] is date:
                        values[attname] = datetime(
                            *xldate_as_tuple(value, self._workbook.datemode)
                        ).date()
                    else:
                        values[attname] = rules['type'](value)
                # Пропускаем ошибку конвертации, она детектируется валидатором
                except Exception:  # pylint: disable=W0703
                    pass

    def _get_xls_data(self):
        for sheet in self.get_sheets():
            for row_number, data in enumerate(sheet.get_rows(), 1):
                # Пропуск пустых строк
                if not any(text_type(cell.value).strip() for cell in data):
                    continue

                if row_number - 1 in self.skip_rows:
                    continue
                yield sheet, row_number, data

    def get_rows(self):
        """Возвращает коллекцию валидных строк из файла."""
        super(ExcelParser, self).get_rows()

        validator = self._get_validator(self.rules)

        for sheet, row_number, data in self._get_xls_data():
            row = self._get_row()

            row.sheet_name = sheet.name
            row.row_number = row_number

            row.values = OrderedDict(zip(
                (self._header_config[header] for header in self.headers),
                (cell.value for cell in data)
            ))

            self._prepare_row(row)

            try:
                validator.validate(row)
            except Exception as exception:  # pylint: disable=W0703
                # TODO: error as object?
                self.logger.log(
                    exception, prefix=text_type(row)
                )
            else:
                yield row
