# encoding: utf-8

from __future__ import unicode_literals

from abc import ABCMeta
from abc import abstractmethod
from datetime import date

import six

from .. import ImportException


class RowValidationError(ImportException):
    u"""Ошибка значения ячейки."""


class IValidator(object):
    """Интерфейс валидатора."""

    __metaclass__ = ABCMeta

    @abstractmethod
    def validate(self, row):
        """Выполняет проверку строки."""


class AbstractValidator(IValidator):
    """Абстрактный валидатор строки."""

    def __init__(self, rules):
        self.rules = rules


class Validator(AbstractValidator):
    """Валидатор строки."""

    EMPTY_VALUE_MSG = 'Отсутствует обязательное значение'
    INCORRECT_VALUE_MSG = 'Некорректное значение "{0}"'

    def _validate_required(self, value):
        """Проверка обязательности значения."""
        if not value:
            raise ValueError(self.EMPTY_VALUE_MSG.format(value))

    def _validate_type(self, type_, value):
        """Проверка типа значения."""
        if not isinstance(value, type_):
            raise ValueError(self.INCORRECT_VALUE_MSG.format(value))

    def _validate(self, attname, value, rules):
        if not value and 'required' in rules and not rules['required']:
            return
        if 'required' in rules and rules['required']:
            self._validate_required(value)

        if 'type' in rules:
            self._validate_type(rules['type'], value)

        for validator in rules.get('validators', ()):
            validator(value)

    def validate(self, row):
        values = getattr(row, 'values', {})

        errors = []

        for attname, value in six.iteritems(values):
            rules = self.rules[attname]

            try:
                self._validate(attname, value, rules)

            except Exception as error:
                errors.append(
                    '{0}: {1}'.format(
                        rules['header'], six.text_type(error)
                    )
                )

        if errors:
            raise RowValidationError(
                '; '.join(errors)
            )
